#!/usr/bin/env node
var __async = (__this, __arguments, generator) => {
  return new Promise((resolve, reject) => {
    var fulfilled = (value) => {
      try {
        step(generator.next(value));
      } catch (e) {
        reject(e);
      }
    };
    var rejected = (value) => {
      try {
        step(generator.throw(value));
      } catch (e) {
        reject(e);
      }
    };
    var step = (x) => x.done ? resolve(x.value) : Promise.resolve(x.value).then(fulfilled, rejected);
    step((generator = generator.apply(__this, __arguments)).next());
  });
};

// src/index.ts
import yargs from "yargs";
import { hideBin } from "yargs/helpers";
import axios from "axios";
var BASE_URL = "https://api.binance.com/api/v3";
var COLOURS = {
  green: "\x1B[32;1m",
  red: "\x1B[31;1m",
  yellow: "\x1B[33;1m",
  reset: "\x1B[0m"
};
var argv = yargs(hideBin(process.argv)).default({
  i: "1h",
  l: 50,
  p: "USDT",
  r: 25
}).help("h").alias("h", "help").alias("i", "interval").describe("i", "Interval eg. 15m, 4h, 1d").alias("p", "pair").describe("p", "Coin pairing eg. BTC, ETH, BNB").alias("l", "limit").describe("l", "Number of candlesticks").alias("r", "rows").describe("r", "Graph height measured in rows").parseSync();
var _a;
var coin = (_a = argv._.length ? argv._[0] : "BTC") == null ? void 0 : _a.toString().toUpperCase();
var { i: interval, l: limit, p: pairInput, r: graphHeight } = argv;
var pair = pairInput == null ? void 0 : pairInput.toUpperCase();
var coinDataUrl = `${BASE_URL}/ticker/24hr?symbol=${coin}${pair}`;
var graphDataUrl = `${BASE_URL}/klines?symbol=${coin}${pair}&interval=${interval}&limit=${limit}`;
var fetcher = (url) => __async(void 0, null, function* () {
  return axios.get(url, { timeout: 5e3 }).then((res) => res.data);
});
(() => __async(void 0, null, function* () {
  var _a2, _b;
  try {
    const [coinData, graphData] = yield Promise.all([
      fetcher(coinDataUrl),
      fetcher(graphDataUrl)
    ]);
    if (!coinData || !graphData) {
      console.log("Error fetching data :( Please try again!");
      return;
    }
    const {
      openingPrices,
      closingPrices,
      highestPrices,
      lowestPrices,
      minPrice,
      maxPrice
    } = graphData.reduce(
      (acc, d, i) => {
        const values = d.slice(1, 5).map((v) => parseFloat(v));
        acc.minPrice = Math.min(acc.minPrice, ...values);
        acc.maxPrice = Math.max(acc.maxPrice, ...values);
        const [open, high, low, close] = values;
        acc.openingPrices.push(open);
        acc.highestPrices.push(high);
        acc.lowestPrices.push(low);
        acc.closingPrices.push(close);
        return acc;
      },
      {
        openingPrices: [],
        closingPrices: [],
        highestPrices: [],
        lowestPrices: [],
        minPrice: parseFloat((_a2 = graphData == null ? void 0 : graphData[0]) == null ? void 0 : _a2[3]),
        maxPrice: parseFloat((_b = graphData == null ? void 0 : graphData[0]) == null ? void 0 : _b[2])
      }
    );
    const range = Math.abs(maxPrice - minPrice);
    const ratio = range !== 0 ? graphHeight / range : 1;
    const adjustedMin = Math.round(minPrice * ratio);
    const adjustedMax = Math.round(maxPrice * ratio);
    const seriesLength = graphData.length;
    const labelOffset = 3;
    const height = Math.abs(adjustedMax - adjustedMin);
    const width = seriesLength + labelOffset;
    const padding = "         ";
    const graph = new Array(height + 1);
    for (let i = 0; i <= height; i++) {
      graph[i] = new Array(width);
      for (let j = 0; j < width; j++) {
        graph[i][j] = " ";
      }
    }
    for (let val = adjustedMin; val <= adjustedMax; ++val) {
      const labelValue = height > 0 ? maxPrice - (val - adjustedMin) * range / height : val;
      const decimal = labelValue < 1 ? 4 : labelValue < 1e3 ? 2 : 0;
      const label = (padding + labelValue.toFixed(decimal)).slice(
        -padding.length
      );
      graph[val - adjustedMin][Math.max(labelOffset - label.length, 0)] = label;
      graph[val - adjustedMin][labelOffset - 1] = "\u2524";
    }
    const getPriceYIndex = (price) => Math.round(price * ratio) - adjustedMin;
    for (let x = 0; x < seriesLength; x++) {
      const openingPriceIndex = getPriceYIndex(openingPrices[x]);
      const closingPriceIndex = getPriceYIndex(closingPrices[x]);
      const highestPriceIndex = getPriceYIndex(highestPrices[x]);
      const lowestPriceIndex = getPriceYIndex(lowestPrices[x]);
      const currentColumn = x + labelOffset;
      const colour = openingPriceIndex <= closingPriceIndex ? COLOURS.green : COLOURS.red;
      for (let i = height - highestPriceIndex; i <= height - lowestPriceIndex; ++i) {
        if (i <= height - openingPriceIndex && i >= height - closingPriceIndex || i >= height - openingPriceIndex && i <= height - closingPriceIndex) {
          graph[i][currentColumn] = `${colour}\u2588${COLOURS.reset}`;
        } else {
          graph[i][currentColumn] = `${colour}|${COLOURS.reset}`;
        }
      }
    }
    console.log(graph.map((x) => x.join(" ")).join("\n"));
    console.log(
      `
 ${COLOURS.yellow}${coin}/${pair}\x1B[0m  ${interval} chart`
    );
    const priceChangeColour = parseInt(coinData.priceChangePercent) >= 0 ? COLOURS.green : COLOURS.red;
    console.log(
      `
 The current price for ${coin} is ${priceChangeColour}${coinData.lastPrice} ${pair} (${coinData.priceChangePercent}%)`
    );
  } catch (error) {
    console.log("Oh no! An error occured. \n");
    if (axios.isAxiosError(error)) {
      console.error(`Error: ${error.message} 
`);
    }
    console.log("Please try again!");
  }
}))();
